% :-swi_module(user). 
:-swi_module(modHelp, [actHelp/0]).
/* * module * A command to tell an agent all the possible commands
% help.pl
% Douglas Miles 2014
*/
:- include(prologmud(mud_header)).

% :- register_module_type (mtCommand).



%=autodoc
%% isa( ?ARG1, ?ARG2) is semidet.
%
%  (isa/2).
%
isa(tHumanControlled,ttAgentType).
%genls(ttAgentType,tCol).

:- kb_global(baseKB:type_action_info/3).



%=autodoc
%% type_action_info( ?ARG1, ?ARG2, ?ARG3) is semidet.
%
% Type Action Info.
%
baseKB:type_action_info(tHumanControlled,actHelp(isOptional(ftString,"")), "shows this help").


:-export(get_all_templates/1).



%=autodoc
%% get_all_templates( ?Templ) is semidet.
%
% Get All Templates.
%
get_all_templates(Templ):- no_repeats(get_all_templates0(Templ)).

:-export(good_template/1).


%=autodoc
%% good_template( ?Templ) is semidet.
%
% Good Template.
%
good_template(Templ):- \+ contains_singletons(Templ).

get_all_templates0(Templ):-get_good_templates(Templ).
get_all_templates0(Templ):-get_bad_templates(Templ),not(get_good_templates(Templ)).



%=autodoc
%% get_good_templates( ?Templ) is semidet.
%
% Get Good Templates.
%
get_good_templates(Templ):- isa(Templ,vtActionTemplate),good_template(Templ).
% get_good_templates(Templ):- no_repeats_old((baseKB:action_info(Templ,_),good_template(Templ))).




%=autodoc
%% get_bad_templates( ?Templ) is semidet.
%
% Get Bad Templates.
%
get_bad_templates(Templ):- no_repeats_old((baseKB:action_info(Templ,_),not(good_template(Templ)))).


:- sanity((fully_expand_real(foo,baseKB:action_info(TEMPL, txtConcatFn(_Text,"does: ",do(_A2,TEMPL))),O),dmsg(O))).

% :- mpred_core:import(baseKB:get_agent_text_command_0/4).

/*
==> ((({between(1,5,L),length(Text,L),
     get_agent_text_command(_A,Text,A2,Goal),(ground(Goal)->TEMPL=Goal;TEMPL=Text)}==>
         baseKB:action_info(TEMPL, txtConcatFn(Text,"does: ",do(A2,TEMPL)))))).
*/

(action_rules(_Agent,Verb,[Obj|Objs],List),{atomic(Verb),safe_univ(Syntax,[Verb,Obj|Objs])} ==> 
         baseKB:action_info(Syntax, txtConcatFn(["makes","happen"|List]))).

%=autodoc
%%  ?Sub==> ?Super is semidet.
%
% ==>.
%





%=autodoc
%% to_param_doc( ?TEMPL, ?Prolog) is semidet.
%
% Converted To Param Document.
%
to_param_doc(TEMPL,["Prolog", "looks", "like", ":",TEMPL]):-!.
to_param_doc(TEMPL,S):-sformat(S,'Prolog looks like: ~q',[TEMPL]).




%=autodoc
%% first_pl( ?BODY, ?PL) is semidet.
%
% First Pl.
%
first_pl((BODY,_),PL):- nonvar(BODY),!,
 first_pl(BODY,PL).
first_pl(PL,PL).

:- kb_shared(action_info_db/3).



%=autodoc
%% action_info_db( ?TEMPL, +INFO, ?WAS) is semidet.
%
% Action Info Database.
%
action_info_db(TEMPL,INFO,WAS):- (PRED=baseKB:agent_call_command(_,WAS);PRED=baseKB:agent_text_command(_,_,_,WAS)) ,
   clause(PRED,BODY,REF),clause_property(REF,file(S)),
   (ground(WAS)->true;once(( ignore((nop(S=S),first_pl(BODY,PL),ignore(catch(((true;quietly(PL)),!),_,true)))),ground(WAS)))),
   
    (TEMPL=@=WAS -> ((clause_property(REF,line_count(LC)),INFO=line(LC:S))) ;  (not(not(TEMPL=WAS)) -> INFO=file(S) ; fail)).

% :-trace.
action_info_db(TEMPL,S,WAS) ==> if_missing(baseKB:action_info(TEMPL,_Help), baseKB:action_info(TEMPL,txtConcatFn(S,contains,WAS))).
% baseKB:action_info(TEMPL,txtConcatFn(S,contains,WAS)) <= action_info_db(TEMPL,S,WAS),{not_asserted(baseKB:action_info(TEMPL,_Help))}.




%=autodoc
%% commands_list( ?ListS) is semidet.
%
% Commands List.
%
commands_list(ListS):- findall(Templ,get_all_templates(Templ),List),predsort(alpha_shorter_1,List,ListS).



%=autodoc
%% alpha_shorter( +OrderO, ?P1, ?P2) is semidet.
%
% Alpha Shorter.
%
alpha_shorter(OrderO, P1,P2):-arg(1,P1,O1),arg(1,P2,O2),!,alpha_shorter_1(OrderO, O1,O2),!.
alpha_shorter(OrderO, P1,P2):-alpha_shorter_1(OrderO, P1,P2),!.

alpha_shorter_1(OrderO, P1,P2):-functor_h(P1,F1,A1),functor_h(P2,F2,A2),compare(OrderF,F1,F2), 
 (OrderF \== '=' -> OrderO=OrderF ;
  (compare(OrderA,A1,A2), (OrderA \== '=' -> OrderO=OrderA ; compare(OrderO,P1,P2)))).




%=autodoc
%% show_templ_doc( ?TEMPL) is semidet.
%
% Show Templ Document.
%
show_templ_doc(TEMPL):-findall(DOC,baseKB:action_info(TEMPL,DOC),DOCL),nvfmt(TEMPL=DOCL).


%=autodoc
%% show_templ_doc_all( ?TEMPL) is semidet.
%
% Show Templ Document All.
%
show_templ_doc_all(TEMPL):-findall(DOC,baseKB:action_info(TEMPL,DOC),DOCL),nvfmt(TEMPL=DOCL).



%=autodoc
%% nvfmt( ?XX) is semidet.
%
% Nvfmt.
%
nvfmt([XX]):-!,nvfmt(XX).
nvfmt(XX=[YY]):-!,nvfmt(XX=YY).
nvfmt(XX):-copy_term(XX,X),numbervars(X,0,_,[attvar(bind),singletons(true)]),fmt(X).

% Help - A command to tell an agent all the possible commands
actHelp:- commands_list(ListS),forall(member(E,ListS),show_templ_doc(E)).

%=autodoc
%% actHelp is semidet.
%
% Single Doer Action Help.
%



%=autodoc
%% agent_call_command( ?Agent, ?ACT) is semidet.
%
% Agent Call Command.
%
baseKB:agent_call_command(_Agent,actHelp) :- actHelp.
baseKB:agent_call_command(_Agent,actHelp(Str)) :-actHelp(Str).



%=autodoc
%% actHelp( ?Str) is semidet.
%
% Single Doer Action Help.
%
actHelp(Str):-commands_list(ListS),forall(member(E,ListS),write_doc_if_contains(Str,E)).



%=autodoc
%% write_doc_if_contains( ?, ?E) is semidet.
%
% Write Document If Contains.
%
write_doc_if_contains('',E):-!,show_templ_doc(E),!.
write_doc_if_contains([],E):-!,show_templ_doc(E),!.
write_doc_if_contains("",E):-!,show_templ_doc(E),!.
write_doc_if_contains(Must,E):-ignore((with_output_to(string(Str),show_templ_doc_all(E)),str_contains_all([Must],Str),fmt(Str))).


(vtActionTemplate(A),{nonvar(A),get_functor(A,Inst)} ==> isa(Inst,vtVerb)).



%=autodoc
%% impl_coerce_hook( ?Text, ?VtVerb, +Inst) is semidet.
%
% Implimentation Coerce Hook.
%
impl_coerce_hook(Text,vtVerb,Inst):- isa(Inst,vtVerb),name_text(Inst,Text).

%baseKB:agent_text_command(Agent,[Who],Agent,Cmd):- nonvar(Who), get_all_templates(Syntax),Syntax=..[Who,isOptional(_,Default)],Cmd=..[Who,Default].
%baseKB:agent_text_command(Agent,[Who,Type],Agent,Cmd):- get_all_templates(Syntax),nonvar(Who),Syntax=..[Who,isOptional(Type,_)],Cmd=..[Who,Type].

:- include(prologmud(mud_footer)).

% :-ain((({get_all_templates(Templ)})==>vtActionTemplate(Templ))).

:- kb_shared(baseKB:actParse/2).

(baseKB:type_action_info(_,TEMPL,Help) ==> baseKB:action_info(TEMPL,Help)).

(baseKB:action_info(TEMPL,_Help) ==> vtActionTemplate(TEMPL)).

vtActionTemplate(TEMPL), \+ baseKB:action_info(TEMPL,_)  ==> ({to_param_doc(TEMPL,S)},baseKB:action_info(TEMPL,S)).




